package org.fuseri.moduleexercise.exercise;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.PositiveOrZero;
import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

/**
 * Represent a REST API controller for exercises
 * Handle HTTP requests related to exercises
 */
@RestController
@RequestMapping("/exercises")
public class ExerciseController {

    private final ExerciseFacade facade;

    /**
     * Constructor for AnswerController
     *
     * @param facade the facade responsible for handling exercise-related logic
     */
    @Autowired
    public ExerciseController(ExerciseFacade facade) {
        this.facade = facade;
    }

    /**
     * Create a new answer for the given question ID
     *
     * @param dto the ExerciseCreateDto object containing information about the exercise to create
     * @return an ExerciseDto object representing the newly created exercise
     */
    @PostMapping
    public ExerciseDto create(@Valid @RequestBody ExerciseCreateDto dto) {
        return facade.create(dto);
    }

    /**
     * Find an exercise by ID
     *
     * @param id the ID of the exercise to find
     * @return an ExerciseDto object representing the found exercise
     */
    @GetMapping("/{id}")
    public ExerciseDto find(@NotNull @PathVariable long id) {
        return facade.find(id);
    }

    /**
     * Find exercises and return them in a paginated format
     *
     * @param page the page number of the exercises to retrieve
     * @return a Result object containing a list of ExerciseDto objects and pagination information
     */
    @GetMapping
    public Result<ExerciseDto> findAll(@PositiveOrZero @RequestParam int page) {
        return facade.findAll(page);
    }

    /**
     * Find exercises that mach filters and return them in paginated format
     *
     * @param page       the page number of the exercises to retrieve
     * @param courseId   the id of the course to filter by
     * @param difficulty the difficulty level to filter by
     * @return a Result object containing a list of filtered ExerciseDto objects woth pagination information
     */
    @GetMapping("filter")
    public Result<ExerciseDto> findPerDifficultyPerCourse(
            @PositiveOrZero @RequestParam int page, @NotNull @RequestParam long courseId,
            @PositiveOrZero @RequestParam int difficulty) {
        return facade.findPerDifficultyPerCourse(page, courseId, difficulty);
    }

    /**
     * Find questions by exercise ID and return them in a paginated format
     *
     * @param exerciseId the ID of the exercise to find questions for
     * @param page       the page number of the questions to retrieve
     * @return a Result object containing a list of QuestionDto objects and pagination information
     */
    @GetMapping("/{exercise-id}/questions")
    public Result<QuestionDto> findQuestions(@NotNull @PathVariable("exercise-id") long exerciseId,
                                                @PositiveOrZero @RequestParam int page) {
        return facade.getQuestions(exerciseId, page);
    }

    /**
     * Update an exercise with id
     *
     * @param id  the ID of the exercise to update
     * @param dto the ExerciseCreateDto object containing information about the exercise to update
     * @return an ExerciseDto object representing the updated exercise
     * @throws ResponseStatusException invalid exercise id
     */

    @PutMapping("/{id}")
    public ExerciseDto update(@NotNull @PathVariable long id, @Valid @RequestBody ExerciseCreateDto dto) {
        return facade.update(id, dto);
    }

    /**
     * Delete an exercise with exerciseId
     *
     * @param id the ID of the exercise to delete
     */
    @DeleteMapping("/{id}")
    public void delete(@NotNull @PathVariable long id) {
        facade.delete(id);
    }

}
