package org.fuseri.moduleexercise.question;

import jakarta.persistence.EntityNotFoundException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.fuseri.model.dto.exercise.QuestionUpdateDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

/**
 * Represent a REST API controller for questions
 * Handle HTTP requests related to questions
 */
@RestController
@RequestMapping("/questions")
public class QuestionController {

    private final QuestionFacade questionFacade;

    /**
     * Constructor for QuestionController
     *
     * @param questionFacade the service responsible for handling question-related logic
     */
    @Autowired
    public QuestionController(QuestionFacade questionFacade) {
        this.questionFacade = questionFacade;
    }

    /**
     * Find a question by ID.
     *
     * @param id the ID of the question to find
     * @return a QuestionDto object representing the found question
     */
    @GetMapping("/{id}")
    public QuestionDto find(@NotNull @PathVariable long id) {
        return questionFacade.find(id);
    }

    /**
     * Retrieve a list of AnswerDto objects which belong to question with id
     *
     * @param id the ID of the question for which to retrieve answers
     * @return a List of AnswerDto objects
     */
    @GetMapping("/{id}/answers")
    public List<AnswerDto> getQuestionAnswers(@NotNull @PathVariable long id) {
        return questionFacade.getQuestionAnswers(id);
    }

    /**
     * Add a new question to an exercise
     *
     * @param dto a QuestionCreateDto object representing the new question to add
     * @return a QuestionDto object representing the added question
     * @throws ResponseStatusException if the exercise with exerciseId from QuestionCreateDto does not exist
     */
    @PostMapping
    public QuestionDto addQuestion(@Valid @RequestBody QuestionCreateDto dto) {
        try {
            return questionFacade.create(dto);
        } catch (EntityNotFoundException e) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND,
                    e.getMessage());
        }
    }

    /**
     * Update question
     *
     * @param dto a QuestionDto object representing the updated question with correct id
     * @return a QuestionUpdateDto object representing the updated question
     * @throws ResponseStatusException if the question with id doesn't exist or its exercise doesn't exist
     */
    @PutMapping("/{id}")
    public QuestionDto updateQuestion(@NotNull @PathVariable long id, @Valid @RequestBody QuestionUpdateDto dto) {
        try {
            return questionFacade.update(id, dto);
        } catch (IllegalArgumentException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        } catch (EntityNotFoundException e) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, e.getMessage());
        }
    }

    /**
     * Add a new question to an exercise
     *
     * @param id of question to delete
     */
    @DeleteMapping("/{id}")
    public void deleteQuestion(@NotNull @PathVariable long id) {
        questionFacade.delete(id);
    }
}